@@ -949,6 +949,9 @@ sub ContainsString($)
 {
 	my ($e) = @_;
 
+	if (property_matches($e, "flag", ".*STR_NULLTERM.*")) {
+		return 1;
+	}
 	foreach my $l (@{$e->{LEVELS}}) {
 		return 1 if ($l->{TYPE} eq "ARRAY" and $l->{IS_ZERO_TERMINATED});
 	}
