@@ -155,7 +155,7 @@ static struct share_mapping_entry *add_srt(int snum, const char **mappings)
 	 * catia mappings are of the form :
 	 * UNIX char (in 0xnn hex) : WINDOWS char (in 0xnn hex)
 	 *
-	 * multiple mappings are comma seperated in smb.conf
+	 * multiple mappings are comma separated in smb.conf
 	 */
 	for (i=0;mappings[i];i++) {
 		fstrcpy(mapping, mappings[i]);
@@ -222,7 +222,7 @@ static bool init_mappings(connection_struct *conn,
 static NTSTATUS catia_string_replace_allocate(connection_struct *conn,
 					      const char *name_in,
 					      char **mapped_name,
-					      int direction)
+					enum vfs_translate_direction direction)
 {
 	static smb_ucs2_t *tmpbuf = NULL;
 	smb_ucs2_t *ptr;
@@ -393,16 +393,15 @@ static int catia_rename(vfs_handle_struct *handle,
 	}
 
 	/* Setup temporary smb_filename structs. */
-	status = copy_smb_filename(ctx, smb_fname_src, &smb_fname_src_tmp);
-
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	smb_fname_src_tmp = cp_smb_filename(ctx, smb_fname_src);
+	if (smb_fname_src_tmp == NULL) {
+		errno = ENOMEM;
 		goto out;
 	}
 
-	status = copy_smb_filename(ctx, smb_fname_dst, &smb_fname_dst_tmp);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	smb_fname_dst_tmp = cp_smb_filename(ctx, smb_fname_dst);
+	if (smb_fname_dst_tmp == NULL) {
+		errno = ENOMEM;
 		goto out;
 	}
 
@@ -492,9 +491,9 @@ static int catia_unlink(vfs_handle_struct *handle,
 	}
 
 	/* Setup temporary smb_filename structs. */
-	status = copy_smb_filename(talloc_tos(), smb_fname, &smb_fname_tmp);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	smb_fname_tmp = cp_smb_filename(talloc_tos(), smb_fname);
+	if (smb_fname_tmp == NULL) {
+		errno = ENOMEM;
 		return -1;
 	}
 
@@ -550,6 +549,25 @@ static int catia_lchown(vfs_handle_struct *handle,
 	return ret;
 }
 
+static int catia_chmod(vfs_handle_struct *handle, const char *path, mode_t mode)
+{
+	char *name = NULL;
+	NTSTATUS status;
+	int ret;
+
+	status = catia_string_replace_allocate(handle->conn, path,
+					&name, vfs_translate_to_unix);
+	if (!NT_STATUS_IS_OK(status)) {
+		errno = map_errno_from_nt_status(status);
+		return -1;
+	}
+
+	ret = SMB_VFS_NEXT_CHMOD(handle, name, mode);
+	TALLOC_FREE(name);
+
+	return ret;
+}
+
 static int catia_rmdir(vfs_handle_struct *handle,
 		       const char *path)
 {
@@ -628,9 +646,9 @@ static int catia_ntimes(vfs_handle_struct *handle,
 		return -1;
 	}
 
-	status = copy_smb_filename(talloc_tos(), smb_fname, &smb_fname_tmp);
-	if (!NT_STATUS_IS_OK(status)) {
-		errno = map_errno_from_nt_status(status);
+	smb_fname_tmp = cp_smb_filename(talloc_tos(), smb_fname);
+	if (smb_fname_tmp == NULL) {
+		errno = ENOMEM;
 		return -1;
 	}
 
@@ -687,11 +705,17 @@ catia_streaminfo(struct vfs_handle_struct *handle,
 		 struct files_struct *fsp,
 		 const char *path,
 		 TALLOC_CTX *mem_ctx,
-		 unsigned int *num_streams,
-		 struct stream_struct **streams)
+		 unsigned int *_num_streams,
+		 struct stream_struct **_streams)
 {
 	char *mapped_name = NULL;
 	NTSTATUS status;
+	int i;
+	unsigned int num_streams = 0;
+	struct stream_struct *streams = NULL;
+
+	*_num_streams = 0;
+	*_streams = NULL;
 
 	status = catia_string_replace_allocate(handle->conn, path,
 				        &mapped_name, vfs_translate_to_unix);
@@ -701,10 +725,54 @@ catia_streaminfo(struct vfs_handle_struct *handle,
 	}
 
 	status = SMB_VFS_NEXT_STREAMINFO(handle, fsp, mapped_name,
-					 mem_ctx, num_streams,streams);
+					 mem_ctx, &num_streams, &streams);
 	TALLOC_FREE(mapped_name);
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
 
-	return status;
+	/*
+	 * Translate stream names just like the base names
+	 */
+	for (i = 0; i < num_streams; i++) {
+		/*
+		 * Strip ":" prefix and ":$DATA" suffix to get a
+		 * "pure" stream name and only translate that.
+		 */
+		void *old_ptr = streams[i].name;
+		char *stream_name = streams[i].name + 1;
+		char *stream_type = strrchr_m(stream_name, ':');
+
+		if (stream_type != NULL) {
+			*stream_type = '\0';
+			stream_type += 1;
+		}
+
+		status = catia_string_replace_allocate(handle->conn, stream_name,
+						       &mapped_name, vfs_translate_to_windows);
+		if (!NT_STATUS_IS_OK(status)) {
+			TALLOC_FREE(streams);
+			return status;
+		}
+
+		if (stream_type != NULL) {
+			streams[i].name = talloc_asprintf(streams, ":%s:%s",
+							  mapped_name, stream_type);
+		} else {
+			streams[i].name = talloc_asprintf(streams, ":%s",
+							  mapped_name);
+		}
+		TALLOC_FREE(mapped_name);
+		TALLOC_FREE(old_ptr);
+		if (streams[i].name == NULL) {
+			TALLOC_FREE(streams);
+			return NT_STATUS_NO_MEMORY;
+		}
+	}
+
+	*_num_streams = num_streams;
+	*_streams = streams;
+	return NT_STATUS_OK;
 }
 
 static NTSTATUS
@@ -918,6 +986,7 @@ static struct vfs_fn_pointers vfs_catia_fns = {
 	.unlink_fn = catia_unlink,
 	.chown_fn = catia_chown,
 	.lchown_fn = catia_lchown,
+	.chmod_fn = catia_chmod,
 	.chdir_fn = catia_chdir,
 	.ntimes_fn = catia_ntimes,
 	.realpath_fn = catia_realpath,
