@@ -82,7 +82,7 @@ ssize_t read_file(files_struct *fsp,char *data,off_t pos,size_t n)
 		return n;
 	}
 
-	flush_write_cache(fsp, READ_FLUSH);
+	flush_write_cache(fsp, SAMBA_READ_FLUSH);
 
 	fsp->fh->pos = pos;
 
@@ -164,8 +164,8 @@ static int wcp_file_size_change(files_struct *fsp)
 	return ret;
 }
 
-void update_write_time_handler(struct event_context *ctx,
-				      struct timed_event *te,
+void update_write_time_handler(struct tevent_context *ctx,
+				      struct tevent_timer *te,
 				      struct timeval now,
 				      void *private_data)
 {
@@ -339,18 +339,21 @@ ssize_t write_file(struct smb_request *req,
 	if (!fsp->modified &&
 	    EXCLUSIVE_OPLOCK_TYPE(fsp->oplock_type) &&
 	    (wcp == NULL)) {
+		/*
+		 * Note: no write cache with leases!
+		 * as the handles would have to share the write cache
+		 * that's possible but an improvement for another day...
+		 */
 		setup_write_cache(fsp, fsp->fsp_name->st.st_ex_size);
 		wcp = fsp->wcp;
 	}
 
 	mark_file_modified(fsp);
 
-#ifdef WITH_PROFILE
 	DO_PROFILE_INC(writecache_total_writes);
 	if (!fsp->oplock_type) {
 		DO_PROFILE_INC(writecache_non_oplock_writes);
 	}
-#endif
 
 	/*
 	 * If this file is level II oplocked then we need
@@ -364,36 +367,11 @@ ssize_t write_file(struct smb_request *req,
 	contend_level2_oplocks_begin(fsp, LEVEL2_CONTEND_WRITE);
 	contend_level2_oplocks_end(fsp, LEVEL2_CONTEND_WRITE);
 
-#ifdef WITH_PROFILE
-	if (profile_p && profile_p->writecache_total_writes % 500 == 0) {
-		DEBUG(3,("WRITECACHE: initwrites=%u abutted=%u total=%u \
-nonop=%u allocated=%u active=%u direct=%u perfect=%u readhits=%u\n",
-			profile_p->writecache_init_writes,
-			profile_p->writecache_abutted_writes,
-			profile_p->writecache_total_writes,
-			profile_p->writecache_non_oplock_writes,
-			profile_p->writecache_allocated_write_caches,
-			profile_p->writecache_num_write_caches,
-			profile_p->writecache_direct_writes,
-			profile_p->writecache_num_perfect_writes,
-			profile_p->writecache_read_hits ));
-
-		DEBUG(3,("WRITECACHE: Flushes SEEK=%d, READ=%d, WRITE=%d, READRAW=%d, OPLOCK=%d, CLOSE=%d, SYNC=%d\n",
-			profile_p->writecache_flushed_writes[SEEK_FLUSH],
-			profile_p->writecache_flushed_writes[READ_FLUSH],
-			profile_p->writecache_flushed_writes[WRITE_FLUSH],
-			profile_p->writecache_flushed_writes[READRAW_FLUSH],
-			profile_p->writecache_flushed_writes[OPLOCK_RELEASE_FLUSH],
-			profile_p->writecache_flushed_writes[CLOSE_FLUSH],
-			profile_p->writecache_flushed_writes[SYNC_FLUSH] ));
-	}
-#endif
-
 	if (wcp && req->unread_bytes) {
 		/* If we're using receivefile don't
 		 * deal with a write cache.
 		 */
-		flush_write_cache(fsp, WRITE_FLUSH);
+		flush_write_cache(fsp, SAMBA_WRITE_FLUSH);
 		delete_write_cache(fsp);
 		wcp = NULL;
 	}
@@ -689,7 +667,7 @@ nonop=%u allocated=%u active=%u direct=%u perfect=%u readhits=%u\n",
                                  +-----------------------+--------+
                         */
 
-			flush_write_cache(fsp, WRITE_FLUSH);
+			flush_write_cache(fsp, SAMBA_WRITE_FLUSH);
 			wcp->offset = wcp->file_size;
 			wcp->data_size = pos - wcp->file_size + 1;
 			memset(wcp->data, '\0', wcp->data_size);
@@ -793,12 +771,12 @@ cache: fd = %d, off=%.0f, size=%u\n", fsp->fh->fd, (double)wcp->offset, (unsigne
 		}
 
 		if (cache_flush_needed) {
-			DEBUG(3,("WRITE_FLUSH:%d: due to noncontinuous write: fd = %d, size = %.0f, pos = %.0f, \
+			DEBUG(3,("SAMBA_WRITE_FLUSH:%d: due to noncontinuous write: fd = %d, size = %.0f, pos = %.0f, \
 n = %u, wcp->offset=%.0f, wcp->data_size=%u\n",
 				write_path, fsp->fh->fd, (double)wcp->file_size, (double)pos, (unsigned int)n,
 				(double)wcp->offset, (unsigned int)wcp->data_size ));
 
-			flush_write_cache(fsp, WRITE_FLUSH);
+			flush_write_cache(fsp, SAMBA_WRITE_FLUSH);
 		}
 	}
 
@@ -826,13 +804,11 @@ n = %u, wcp->offset=%.0f, wcp->data_size=%u\n",
 	 */
 
 	if (n) {
-#ifdef WITH_PROFILE
 		if (wcp->data_size) {
 			DO_PROFILE_INC(writecache_abutted_writes);
 		} else {
 			DO_PROFILE_INC(writecache_init_writes);
 		}
-#endif
 
 		if ((wcp->data_size == 0)
 		    && (pos > wcp->file_size)
@@ -997,11 +973,9 @@ ssize_t flush_write_cache(files_struct *fsp, enum flush_reason_enum reason)
 	DEBUG(9,("flushing write cache: fd = %d, off=%.0f, size=%u\n",
 		fsp->fh->fd, (double)wcp->offset, (unsigned int)data_size));
 
-#ifdef WITH_PROFILE
 	if(data_size == wcp->alloc_size) {
 		DO_PROFILE_INC(writecache_num_perfect_writes);
 	}
-#endif
 
 	ret = real_write_file(NULL, fsp, wcp->data, wcp->offset, data_size);
 
@@ -1026,8 +1000,8 @@ NTSTATUS sync_file(connection_struct *conn, files_struct *fsp, bool write_throug
 		return NT_STATUS_INVALID_HANDLE;
 
 	if (lp_strict_sync(SNUM(conn)) &&
-	    (lp_syncalways(SNUM(conn)) || write_through)) {
-		int ret = flush_write_cache(fsp, SYNC_FLUSH);
+	    (lp_sync_always(SNUM(conn)) || write_through)) {
+		int ret = flush_write_cache(fsp, SAMBA_SYNC_FLUSH);
 		if (ret == -1) {
 			return map_nt_error_from_unix(errno);
 		}
