@@ -29,6 +29,8 @@
 #include "../libcli/registry/util_reg.h"
 #include "libsmb/libsmb.h"
 #include "../libcli/smb/smbXcli_base.h"
+#include "auth/gensec/gensec.h"
+#include "auth/credentials/credentials.h"
 
 /* support itanium as well */
 static const struct print_architecture_table_node archi_table[]= {
@@ -177,7 +179,7 @@ NTSTATUS net_copy_fileattr(struct net_context *c,
 	nt_status = cli_ntcreate(cli_share_src, src_name, 0,
 				 READ_CONTROL_ACCESS, 0,
 				 FILE_SHARE_READ|FILE_SHARE_WRITE, FILE_OPEN,
-				 0x0, 0x0, &fnum_src);
+				 0x0, 0x0, &fnum_src, NULL);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		DEBUGADD(0,("cannot open %s %s on originating server %s\n",
 			is_file?"file":"dir", src_name, nt_errstr(nt_status)));
@@ -214,7 +216,7 @@ NTSTATUS net_copy_fileattr(struct net_context *c,
 	nt_status = cli_ntcreate(cli_share_dst, dst_name, 0,
 				 WRITE_DAC_ACCESS | WRITE_OWNER_ACCESS, 0,
 				 FILE_SHARE_READ|FILE_SHARE_WRITE, FILE_OPEN,
-				 0x0, 0x0, &fnum_dst);
+				 0x0, 0x0, &fnum_dst, NULL);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		DEBUG(0,("failed to open %s on the destination server: %s: %s\n",
 			is_file?"file":"dir", dst_name, nt_errstr(nt_status)));
@@ -331,7 +333,8 @@ NTSTATUS net_copy_file(struct net_context *c,
 		nt_status = cli_open(cli_share_src, src_name, O_RDONLY, DENY_NONE, &fnum_src);
 	else
 		nt_status = cli_ntcreate(cli_share_src, src_name, 0, READ_CONTROL_ACCESS, 0,
-				FILE_SHARE_READ|FILE_SHARE_WRITE, FILE_OPEN, 0x0, 0x0, &fnum_src);
+				FILE_SHARE_READ|FILE_SHARE_WRITE,
+				FILE_OPEN, 0x0, 0x0, &fnum_src, NULL);
 
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		DEBUGADD(0,("cannot open %s %s on originating server %s\n",
@@ -703,9 +706,10 @@ static bool net_spoolss_open_printer_ex(struct rpc_pipe_client *pipe_hnd,
 					TALLOC_CTX *mem_ctx,
 					const char *printername,
 					uint32_t access_required,
-					const char *username,
 					struct policy_handle *hnd)
 {
+	struct cli_credentials *creds = gensec_get_credentials(pipe_hnd->auth->auth_ctx);
+	const char *username = cli_credentials_get_username(creds);
 	WERROR result;
 	fstring printername2;
 
@@ -1135,7 +1139,6 @@ static bool get_printer_info(struct rpc_pipe_client *pipe_hnd,
 	/* argument given, get a single printer by name */
 	if (!net_spoolss_open_printer_ex(pipe_hnd, mem_ctx, argv[0],
 					 MAXIMUM_ALLOWED_ACCESS,
-					 pipe_hnd->auth->user_name,
 					 &hnd))
 		return false;
 
@@ -1329,7 +1332,7 @@ static NTSTATUS rpc_printer_publish_internals_args(struct rpc_pipe_client *pipe_
 
 		/* open printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd, mem_ctx, sharename,
-			PRINTER_ALL_ACCESS, pipe_hnd->auth->user_name, &hnd))
+			PRINTER_ALL_ACCESS, &hnd))
 			goto done;
 
 		/* check for existing dst printer */
@@ -1486,7 +1489,7 @@ NTSTATUS rpc_printer_publish_list_internals(struct net_context *c,
 
 		/* open printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd, mem_ctx, sharename,
-			PRINTER_ALL_ACCESS, cli->user_name, &hnd))
+			PRINTER_ALL_ACCESS, &hnd))
 			goto done;
 
 		/* check for existing dst printer */
@@ -1577,7 +1580,7 @@ NTSTATUS rpc_printer_migrate_security_internals(struct net_context *c,
 
 	/* connect destination PI_SPOOLSS */
 	nt_status = connect_dst_pipe(c, &cli_dst, &pipe_hnd_dst,
-				     &ndr_table_spoolss.syntax_id);
+				     &ndr_table_spoolss);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		return nt_status;
 	}
@@ -1622,12 +1625,12 @@ NTSTATUS rpc_printer_migrate_security_internals(struct net_context *c,
 
 		/* open src printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd, mem_ctx, sharename,
-			MAXIMUM_ALLOWED_ACCESS, cli->user_name, &hnd_src))
+			MAXIMUM_ALLOWED_ACCESS, &hnd_src))
 			goto done;
 
 		/* open dst printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd_dst, mem_ctx, sharename,
-			PRINTER_ALL_ACCESS, cli_dst->user_name, &hnd_dst))
+			PRINTER_ALL_ACCESS, &hnd_dst))
 			goto done;
 
 		/* check for existing dst printer */
@@ -1642,7 +1645,17 @@ NTSTATUS rpc_printer_migrate_security_internals(struct net_context *c,
 
 		/* copy secdesc (info level 2) */
 		info_dst.info2.devmode = NULL;
-		info_dst.info2.secdesc = dup_sec_desc(mem_ctx, info_src.info3.secdesc);
+		if (info_src.info3.secdesc == NULL) {
+			info_dst.info2.secdesc = NULL;
+		} else {
+			info_dst.info2.secdesc
+				= security_descriptor_copy(mem_ctx,
+							info_src.info3.secdesc);
+			if (info_dst.info2.secdesc == NULL) {
+				nt_status = NT_STATUS_NO_MEMORY;
+				goto done;
+			}
+		}
 
 		if (c->opt_verbose)
 			display_sec_desc(info_dst.info2.secdesc);
@@ -1729,7 +1742,7 @@ NTSTATUS rpc_printer_migrate_forms_internals(struct net_context *c,
 
 	/* connect destination PI_SPOOLSS */
 	nt_status = connect_dst_pipe(c, &cli_dst, &pipe_hnd_dst,
-				     &ndr_table_spoolss.syntax_id);
+				     &ndr_table_spoolss);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		return nt_status;
 	}
@@ -1768,12 +1781,12 @@ NTSTATUS rpc_printer_migrate_forms_internals(struct net_context *c,
 
 		/* open src printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd, mem_ctx, sharename,
-			MAXIMUM_ALLOWED_ACCESS, cli->user_name, &hnd_src))
+			MAXIMUM_ALLOWED_ACCESS, &hnd_src))
 			goto done;
 
 		/* open dst printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd_dst, mem_ctx, sharename,
-			PRINTER_ALL_ACCESS, cli->user_name, &hnd_dst))
+			PRINTER_ALL_ACCESS, &hnd_dst))
 			goto done;
 
 		/* check for existing dst printer */
@@ -1906,7 +1919,7 @@ NTSTATUS rpc_printer_migrate_drivers_internals(struct net_context *c,
 	DEBUG(3,("copying printer-drivers\n"));
 
 	nt_status = connect_dst_pipe(c, &cli_dst, &pipe_hnd_dst,
-				     &ndr_table_spoolss.syntax_id);
+				     &ndr_table_spoolss);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		return nt_status;
 	}
@@ -1968,7 +1981,7 @@ NTSTATUS rpc_printer_migrate_drivers_internals(struct net_context *c,
 
 		/* open dst printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd_dst, mem_ctx, sharename,
-			PRINTER_ALL_ACCESS, cli->user_name, &hnd_dst))
+			PRINTER_ALL_ACCESS, &hnd_dst))
 			goto done;
 
 		/* check for existing dst printer */
@@ -1979,7 +1992,6 @@ NTSTATUS rpc_printer_migrate_drivers_internals(struct net_context *c,
 		/* open src printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd, mem_ctx, sharename,
 						 MAXIMUM_ALLOWED_ACCESS,
-						 pipe_hnd->auth->user_name,
 						 &hnd_src))
 			goto done;
 
@@ -2019,7 +2031,7 @@ NTSTATUS rpc_printer_migrate_drivers_internals(struct net_context *c,
 				goto done;
 			}
 
-			DEBUGADD(1,("Sucessfully added driver [%s] for printer [%s]\n",
+			DEBUGADD(1,("Successfully added driver [%s] for printer [%s]\n",
 				drivername, printername));
 
 		}
@@ -2038,7 +2050,7 @@ NTSTATUS rpc_printer_migrate_drivers_internals(struct net_context *c,
 			goto done;
 		}
 
-		DEBUGADD(1,("Sucessfully set driver %s for printer %s\n",
+		DEBUGADD(1,("Successfully set driver %s for printer %s\n",
 			drivername, printername));
 
 		/* close dst */
@@ -2125,7 +2137,7 @@ NTSTATUS rpc_printer_migrate_printers_internals(struct net_context *c,
 
 	/* connect destination PI_SPOOLSS */
 	nt_status = connect_dst_pipe(c, &cli_dst, &pipe_hnd_dst,
-				     &ndr_table_spoolss.syntax_id);
+				     &ndr_table_spoolss);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		return nt_status;
 	}
@@ -2165,7 +2177,7 @@ NTSTATUS rpc_printer_migrate_printers_internals(struct net_context *c,
 
 		/* open dst printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd_dst, mem_ctx, sharename,
-			PRINTER_ALL_ACCESS, cli->user_name, &hnd_dst)) {
+			PRINTER_ALL_ACCESS, &hnd_dst)) {
 
 			DEBUG(1,("could not open printer: %s\n", sharename));
 		}
@@ -2187,7 +2199,7 @@ NTSTATUS rpc_printer_migrate_printers_internals(struct net_context *c,
 
 		/* open src printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd, mem_ctx, sharename,
-			MAXIMUM_ALLOWED_ACCESS, cli->user_name, &hnd_src))
+			MAXIMUM_ALLOWED_ACCESS, &hnd_src))
 			goto done;
 
 		/* getprinter on the src server */
@@ -2300,7 +2312,7 @@ NTSTATUS rpc_printer_migrate_settings_internals(struct net_context *c,
 
 	/* connect destination PI_SPOOLSS */
 	nt_status = connect_dst_pipe(c, &cli_dst, &pipe_hnd_dst,
-				     &ndr_table_spoolss.syntax_id);
+				     &ndr_table_spoolss);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		return nt_status;
 	}
@@ -2352,12 +2364,12 @@ NTSTATUS rpc_printer_migrate_settings_internals(struct net_context *c,
 
 		/* open src printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd, mem_ctx, sharename,
-			MAXIMUM_ALLOWED_ACCESS, cli->user_name, &hnd_src))
+			MAXIMUM_ALLOWED_ACCESS, &hnd_src))
 			goto done;
 
 		/* open dst printer handle */
 		if (!net_spoolss_open_printer_ex(pipe_hnd_dst, mem_ctx, sharename,
-			PRINTER_ALL_ACCESS, cli_dst->user_name, &hnd_dst))
+			PRINTER_ALL_ACCESS, &hnd_dst))
 			goto done;
 
 		/* check for existing dst printer */
