@@ -37,6 +37,9 @@
 #include "dsdb/samdb/ldb_modules/util.h"
 #include "dsdb/samdb/samdb.h"
 #include "librpc/ndr/libndr.h"
+#include "auth/credentials/credentials.h"
+#include "param/secrets.h"
+#include "lib/ldb-samba/ldb_wrap.h"
 
 static int read_at_rootdse_record(struct ldb_context *ldb, struct ldb_module *module, TALLOC_CTX *mem_ctx,
 				  struct ldb_message **msg, struct ldb_request *parent)
@@ -102,9 +105,11 @@ static int prepare_modules_line(struct ldb_context *ldb,
 	}
 
 	if (backend_mod) {
-		backend_full_list = (const char **)str_list_make_single(tmp_ctx, backend_mod);
+		char **b = str_list_make_single(tmp_ctx, backend_mod);
+		backend_full_list = discard_const_p(const char *, b);
 	} else {
-		backend_full_list = (const char **)str_list_make_empty(tmp_ctx);
+		char **b = str_list_make_empty(tmp_ctx);
+		backend_full_list = discard_const_p(const char *, b);
 	}
 	if (!backend_full_list) {
 		talloc_free(tmp_ctx);
@@ -129,7 +134,99 @@ static int prepare_modules_line(struct ldb_context *ldb,
 	return ret;
 }
 
+/*
+ * Force overwrite of the credentials with those
+ * specified in secrets.ldb, to connect across the
+ * ldapi socket to an LDAP backend
+ */
 
+static int set_ldap_credentials(struct ldb_context *ldb, bool use_external)
+{
+	const char *secrets_ldb_path, *sam_ldb_path;
+	char *private_dir, *p, *error_string;
+	struct ldb_context *secrets_ldb;
+	struct cli_credentials *cred;
+	struct loadparm_context *lp_ctx = ldb_get_opaque(ldb, "loadparm");
+	TALLOC_CTX *tmp_ctx = talloc_new(ldb);
+
+	if (!tmp_ctx) {
+		return ldb_oom(ldb);
+	}
+
+	cred = cli_credentials_init(ldb);
+	if (!cred) {
+		talloc_free(tmp_ctx);
+		return ldb_oom(ldb);
+	}
+	cli_credentials_set_anonymous(cred);
+	if (use_external) {
+		cli_credentials_set_forced_sasl_mech(cred, "EXTERNAL");
+	} else {
+		cli_credentials_set_forced_sasl_mech(cred, "DIGEST-MD5");
+
+		/*
+		 * We don't want to use krb5 to talk to our samdb - recursion
+		 * here would be bad, and this account isn't in the KDC
+		 * anyway
+		 */
+		cli_credentials_set_kerberos_state(cred, CRED_DONT_USE_KERBEROS);
+
+		/*
+		 * Work out where *our* secrets.ldb is.  It must be in
+		 * the same directory as sam.ldb
+		 */
+		sam_ldb_path = (const char *)ldb_get_opaque(ldb, "ldb_url");
+		if (!sam_ldb_path) {
+			talloc_free(tmp_ctx);
+			return ldb_operr(ldb);
+		}
+		if (strncmp("tdb://", sam_ldb_path, 6) == 0) {
+			sam_ldb_path += 6;
+		}
+		private_dir = talloc_strdup(tmp_ctx, sam_ldb_path);
+		p = strrchr(private_dir, '/');
+		if (p) {
+			*p = '\0';
+		} else {
+			private_dir = talloc_strdup(tmp_ctx, ".");
+		}
+
+		secrets_ldb_path = talloc_asprintf(private_dir, "tdb://%s/secrets.ldb",
+						   private_dir);
+
+		if (!secrets_ldb_path) {
+			talloc_free(tmp_ctx);
+			return ldb_oom(ldb);
+		}
+
+		/*
+		 * Now that we have found the location, connect to
+		 * secrets.ldb so we can read the SamDB Credentials
+		 * record
+		 */
+		secrets_ldb = ldb_wrap_connect(tmp_ctx, NULL, lp_ctx, secrets_ldb_path,
+					       NULL, NULL, 0);
+
+		if (!NT_STATUS_IS_OK(cli_credentials_set_secrets(cred, NULL, secrets_ldb, NULL,
+								 SECRETS_LDAP_FILTER, &error_string))) {
+			ldb_asprintf_errstring(ldb, "Failed to read LDAP backend password from %s", secrets_ldb_path);
+			talloc_free(tmp_ctx);
+			return LDB_ERR_STRONG_AUTH_REQUIRED;
+		}
+	}
+
+	/*
+	 * Finally overwrite any supplied credentials with
+	 * these ones, as only secrets.ldb contains the magic
+	 * credentials to talk on the ldapi socket
+	 */
+	if (ldb_set_opaque(ldb, "credentials", cred)) {
+		talloc_free(tmp_ctx);
+		return ldb_operr(ldb);
+	}
+	talloc_free(tmp_ctx);
+	return LDB_SUCCESS;
+}
 
 static int samba_dsdb_init(struct ldb_module *module)
 {
@@ -137,8 +234,8 @@ static int samba_dsdb_init(struct ldb_module *module)
 	int ret, len, i;
 	TALLOC_CTX *tmp_ctx = talloc_new(module);
 	struct ldb_result *res;
-	struct ldb_message *rootdse_msg, *partition_msg;
-	struct ldb_dn *samba_dsdb_dn;
+	struct ldb_message *rootdse_msg = NULL, *partition_msg;
+	struct ldb_dn *samba_dsdb_dn, *partition_dn;
 	struct ldb_module *backend_module, *module_chain;
 	const char **final_module_list, **reverse_module_list;
 	/*
@@ -162,7 +259,7 @@ static int samba_dsdb_init(struct ldb_module *module)
 	  stack visually - the code below then handles the creation of the list
 	  based on the parameters loaded from the database.
 	*/
-	static const char *modules_list[] = {"resolve_oids",
+	static const char *modules_list1[] = {"resolve_oids",
 					     "rootdse",
 					     "schema_load",
 					     "lazy_commit",
@@ -173,8 +270,9 @@ static int samba_dsdb_init(struct ldb_module *module)
 					     "server_sort",
 					     "asq",
 					     "extended_dn_store",
-					     "extended_dn_in",
-					     "objectclass",
+					     NULL };
+	/* extended_dn_in or extended_dn_in_openldap goes here */
+	static const char *modules_list1a[] = {"objectclass",
 					     "descriptor",
 					     "acl",
 					     "aclread",
@@ -202,6 +300,7 @@ static int samba_dsdb_init(struct ldb_module *module)
 	const char *extended_dn_module_ldb = "extended_dn_out_ldb";
 	const char *extended_dn_module_fds = "extended_dn_out_fds";
 	const char *extended_dn_module_openldap = "extended_dn_out_openldap";
+	const char *extended_dn_in_module = "extended_dn_in";
 
 	static const char *modules_list2[] = {"show_deleted",
 					      "new_partition",
@@ -212,10 +311,12 @@ static int samba_dsdb_init(struct ldb_module *module)
 	static const char *fedora_ds_backend_modules[] = {
 		"nsuniqueid", "paged_searches", "simple_dn", NULL };
 	static const char *openldap_backend_modules[] = {
-		"entryuuid", "paged_searches", "simple_dn", NULL };
+		"entryuuid", "simple_dn", NULL };
 
-	static const char *samba_dsdb_attrs[] = { "backendType", "serverRole", NULL };
-	const char *backendType, *serverRole;
+	static const char *samba_dsdb_attrs[] = { "backendType", NULL };
+	static const char *partition_attrs[] = { "ldapBackend", NULL };
+	const char *backendType, *backendUrl;
+	bool use_sasl_external = false;
 
 	if (!tmp_ctx) {
 		return ldb_oom(ldb);
@@ -233,6 +334,12 @@ static int samba_dsdb_init(struct ldb_module *module)
 		return ldb_oom(ldb);
 	}
 
+	partition_dn = ldb_dn_new(tmp_ctx, ldb, DSDB_PARTITION_DN);
+	if (!partition_dn) {
+		talloc_free(tmp_ctx);
+		return ldb_oom(ldb);
+	}
+
 #define CHECK_LDB_RET(check_ret)				\
 	do {							\
 		if (check_ret != LDB_SUCCESS) {			\
@@ -245,10 +352,8 @@ static int samba_dsdb_init(struct ldb_module *module)
 	                            samba_dsdb_attrs, DSDB_FLAG_NEXT_MODULE, NULL);
 	if (ret == LDB_ERR_NO_SUCH_OBJECT) {
 		backendType = "ldb";
-		serverRole = "domain controller";
 	} else if (ret == LDB_SUCCESS) {
 		backendType = ldb_msg_find_attr_as_string(res->msgs[0], "backendType", "ldb");
-		serverRole = ldb_msg_find_attr_as_string(res->msgs[0], "serverRole", "domain controller");
 	} else {
 		talloc_free(tmp_ctx);
 		return ret;
@@ -259,6 +364,20 @@ static int samba_dsdb_init(struct ldb_module *module)
 		extended_dn_module = extended_dn_module_ldb;
 		link_modules = tdb_modules_list;
 	} else {
+		struct cli_credentials *cred;
+		bool is_ldapi = false;
+
+		ret = dsdb_module_search_dn(module, tmp_ctx, &res, partition_dn,
+					    partition_attrs, DSDB_FLAG_NEXT_MODULE, NULL);
+		if (ret == LDB_SUCCESS) {
+			backendUrl = ldb_msg_find_attr_as_string(res->msgs[0], "ldapBackend", "ldapi://");
+			if (!strncasecmp(backendUrl, "ldapi://", sizeof("ldapi://")-1)) {
+				is_ldapi = true;
+			}
+		} else if (ret != LDB_ERR_NO_SUCH_OBJECT) {
+			talloc_free(tmp_ctx);
+			return ret;
+		}
 		if (strcasecmp(backendType, "fedora-ds") == 0) {
 			link_modules = fedora_ds_modules;
 			backend_modules = fedora_ds_backend_modules;
@@ -267,6 +386,10 @@ static int samba_dsdb_init(struct ldb_module *module)
 			link_modules = openldap_modules;
 			backend_modules = openldap_backend_modules;
 			extended_dn_module = extended_dn_module_openldap;
+			extended_dn_in_module = "extended_dn_in_openldap";
+			if (is_ldapi) {
+				use_sasl_external = true;
+			}
 		} else {
 			return ldb_error(ldb, LDB_ERR_OPERATIONS_ERROR, "invalid backend type");
 		}
@@ -274,6 +397,14 @@ static int samba_dsdb_init(struct ldb_module *module)
 		if (ret != LDB_SUCCESS) {
 			ldb_set_errstring(ldb, "Failed to set readOnlySchema opaque");
 		}
+
+		cred = ldb_get_opaque(ldb, "credentials");
+		if (!cred || !cli_credentials_authentication_requested(cred)) {
+			ret = set_ldap_credentials(ldb, use_sasl_external);
+			if (ret != LDB_SUCCESS) {
+				return ret;
+			}
+		}
 	}
 
 #define CHECK_MODULE_LIST \
@@ -284,7 +415,13 @@ static int samba_dsdb_init(struct ldb_module *module)
 		}						\
 	} while (0)
 
-	final_module_list = str_list_copy_const(tmp_ctx, modules_list);
+	final_module_list = str_list_copy_const(tmp_ctx, modules_list1);
+	CHECK_MODULE_LIST;
+
+	final_module_list = str_list_add_const(final_module_list, extended_dn_in_module);
+	CHECK_MODULE_LIST;
+
+	final_module_list = str_list_append_const(final_module_list, modules_list1a);
 	CHECK_MODULE_LIST;
 
 	final_module_list = str_list_append_const(final_module_list, link_modules);
