@@ -1,20 +1,20 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
 
    test suite for schannel operations
 
    Copyright (C) Andrew Tridgell 2004
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -24,15 +24,14 @@
 #include "librpc/gen_ndr/ndr_lsa_c.h"
 #include "librpc/gen_ndr/ndr_samr_c.h"
 #include "auth/credentials/credentials.h"
-#include "torture/rpc/rpc.h"
+#include "torture/rpc/torture_rpc.h"
 #include "lib/cmdline/popt_common.h"
-#include "auth/gensec/schannel_proto.h"
+#include "../libcli/auth/schannel.h"
 #include "libcli/auth/libcli_auth.h"
 #include "libcli/security/security.h"
 #include "system/filesys.h"
 #include "param/param.h"
 #include "librpc/rpc/dcerpc_proto.h"
-#include "auth/gensec/gensec.h"
 #include "libcli/composite/composite.h"
 #include "lib/events/events.h"
 
@@ -41,8 +40,8 @@
 /*
   try a netlogon SamLogon
 */
-bool test_netlogon_ex_ops(struct dcerpc_pipe *p, struct torture_context *tctx, 
-			  struct cli_credentials *credentials, 
+bool test_netlogon_ex_ops(struct dcerpc_pipe *p, struct torture_context *tctx,
+			  struct cli_credentials *credentials,
 			  struct netlogon_creds_CredentialState *creds)
 {
 	NTSTATUS status;
@@ -55,33 +54,42 @@ bool test_netlogon_ex_ops(struct dcerpc_pipe *p, struct torture_context *tctx,
 	DATA_BLOB names_blob, chal, lm_resp, nt_resp;
 	int i;
 	int flags = CLI_CRED_NTLM_AUTH;
-	if (lp_client_lanman_auth(tctx->lp_ctx)) {
+	struct dcerpc_binding_handle *b = p->binding_handle;
+
+	struct netr_UserSessionKey key;
+	struct netr_LMSessionKey LMSessKey;
+	uint32_t validation_levels[] = { 2, 3 };
+	struct netr_SamBaseInfo *base;
+	const char *crypto_alg = "";
+	bool can_do_validation_6 = true;
+
+	if (lpcfg_client_lanman_auth(tctx->lp_ctx)) {
 		flags |= CLI_CRED_LANMAN_AUTH;
 	}
 
-	if (lp_client_ntlmv2_auth(tctx->lp_ctx)) {
+	if (lpcfg_client_ntlmv2_auth(tctx->lp_ctx)) {
 		flags |= CLI_CRED_NTLMv2_AUTH;
 	}
 
-	cli_credentials_get_ntlm_username_domain(cmdline_credentials, tctx, 
+	cli_credentials_get_ntlm_username_domain(cmdline_credentials, tctx,
 						 &ninfo.identity_info.account_name.string,
 						 &ninfo.identity_info.domain_name.string);
-	
-	generate_random_buffer(ninfo.challenge, 
+
+	generate_random_buffer(ninfo.challenge,
 			       sizeof(ninfo.challenge));
-	chal = data_blob_const(ninfo.challenge, 
+	chal = data_blob_const(ninfo.challenge,
 			       sizeof(ninfo.challenge));
 
-	names_blob = NTLMv2_generate_names_blob(tctx, cli_credentials_get_workstation(credentials), 
+	names_blob = NTLMv2_generate_names_blob(tctx, cli_credentials_get_workstation(credentials),
 						cli_credentials_get_domain(credentials));
 
-	status = cli_credentials_get_ntlm_response(cmdline_credentials, tctx, 
-						   &flags, 
+	status = cli_credentials_get_ntlm_response(cmdline_credentials, tctx,
+						   &flags,
 						   chal,
 						   names_blob,
 						   &lm_resp, &nt_resp,
 						   NULL, NULL);
-	torture_assert_ntstatus_ok(tctx, status, 
+	torture_assert_ntstatus_ok(tctx, status,
 				   "cli_credentials_get_ntlm_response failed");
 
 	ninfo.lm.data = lm_resp.data;
@@ -99,22 +107,120 @@ bool test_netlogon_ex_ops(struct dcerpc_pipe *p, struct torture_context *tctx,
 
 	r.in.server_name = talloc_asprintf(tctx, "\\\\%s", dcerpc_server_name(p));
 	r.in.computer_name = cli_credentials_get_workstation(credentials);
-	r.in.logon_level = 2;
+	r.in.logon_level = NetlogonNetworkInformation;
 	r.in.logon= &logon;
 	r.in.flags = &_flags;
 	r.out.validation = &validation;
 	r.out.authoritative = &authoritative;
 	r.out.flags = &_flags;
 
-	torture_comment(tctx, 
-			"Testing LogonSamLogonEx with name %s\n", 
-			ninfo.identity_info.account_name.string);
-	
-	for (i=2;i<3;i++) {
-		r.in.validation_level = i;
-		
-		status = dcerpc_netr_LogonSamLogonEx(p, tctx, &r);
-		torture_assert_ntstatus_ok(tctx, status, "LogonSamLogon failed");
+	/*
+	- retrieve level6
+	- save usrsession and lmsession key
+	- retrieve level 2
+	- calculate, compare
+	- retrieve level 3
+	- calculate, compare
+	*/
+
+	if (creds) {
+		if (creds->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
+			crypto_alg = "AES";
+		} else if (creds->negotiate_flags & NETLOGON_NEG_ARCFOUR) {
+			crypto_alg = "ARCFOUR";
+		}
+	}
+
+	r.in.validation_level = 6;
+
+	torture_comment(tctx,
+			"Testing LogonSamLogonEx with name %s using %s and validation_level: %d\n",
+			ninfo.identity_info.account_name.string, crypto_alg,
+			r.in.validation_level);
+
+	torture_assert_ntstatus_ok(tctx,
+		dcerpc_netr_LogonSamLogonEx_r(b, tctx, &r),
+		"LogonSamLogonEx failed");
+	if (NT_STATUS_EQUAL(r.out.result, NT_STATUS_INVALID_INFO_CLASS)) {
+		can_do_validation_6 = false;
+	} else {
+		torture_assert_ntstatus_ok(tctx, r.out.result,
+			"LogonSamLogonEx failed");
+
+		key = r.out.validation->sam6->base.key;
+		LMSessKey = r.out.validation->sam6->base.LMSessKey;
+
+		DEBUG(1,("unencrypted session keys from validation_level 6:\n"));
+		dump_data(1, r.out.validation->sam6->base.key.key, 16);
+		dump_data(1, r.out.validation->sam6->base.LMSessKey.key, 8);
+	}
+
+	for (i=0; i < ARRAY_SIZE(validation_levels); i++) {
+
+		r.in.validation_level = validation_levels[i];
+
+		torture_comment(tctx,
+			"Testing LogonSamLogonEx with name %s using %s and validation_level: %d\n",
+			ninfo.identity_info.account_name.string, crypto_alg,
+			r.in.validation_level);
+
+		torture_assert_ntstatus_ok(tctx,
+			dcerpc_netr_LogonSamLogonEx_r(b, tctx, &r),
+			"LogonSamLogonEx failed");
+		torture_assert_ntstatus_ok(tctx, r.out.result,
+			"LogonSamLogonEx failed");
+
+		if (creds == NULL) {
+			/* when this test is called without creds no point in
+			 * testing the session keys */
+			continue;
+		}
+
+		switch (validation_levels[i]) {
+		case 2:
+			base = &r.out.validation->sam2->base;
+			break;
+		case 3:
+			base = &r.out.validation->sam3->base;
+			break;
+		default:
+			break;
+		}
+
+		DEBUG(1,("encrypted keys validation_level %d:\n",
+			validation_levels[i]));
+		dump_data(1, base->key.key, 16);
+		dump_data(1, base->LMSessKey.key, 8);
+
+		if (creds->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
+			netlogon_creds_aes_decrypt(creds, base->key.key, 16);
+			netlogon_creds_aes_decrypt(creds, base->LMSessKey.key, 8);
+		} else if (creds->negotiate_flags & NETLOGON_NEG_ARCFOUR) {
+			netlogon_creds_arcfour_crypt(creds, base->key.key, 16);
+			netlogon_creds_arcfour_crypt(creds, base->LMSessKey.key, 8);
+		}
+
+		DEBUG(1,("decryped keys validation_level %d\n",
+			validation_levels[i]));
+
+		dump_data(1, base->key.key, 16);
+		dump_data(1, base->LMSessKey.key, 8);
+
+		if (!can_do_validation_6) {
+			/* we cant compare against unencrypted keys */
+			continue;
+		}
+
+		torture_assert_mem_equal(tctx,
+					 base->key.key,
+					 key.key,
+					 16,
+					 "unexpected user session key\n");
+		torture_assert_mem_equal(tctx,
+					 base->LMSessKey.key,
+					 LMSessKey.key,
+					 8,
+					 "unexpected LM session key\n");
 	}
 
 	return true;
@@ -124,9 +230,8 @@ bool test_netlogon_ex_ops(struct dcerpc_pipe *p, struct torture_context *tctx,
   do some samr ops using the schannel connection
  */
 static bool test_samr_ops(struct torture_context *tctx,
-			  struct dcerpc_pipe *p)
+			  struct dcerpc_binding_handle *b)
 {
-	NTSTATUS status;
 	struct samr_GetDomPwInfo r;
 	struct samr_PwInfo info;
 	struct samr_Connect connect_r;
@@ -136,23 +241,24 @@ static bool test_samr_ops(struct torture_context *tctx,
 	struct policy_handle handle;
 	struct policy_handle domain_handle;
 
-	name.string = lp_workgroup(tctx->lp_ctx);
+	name.string = lpcfg_workgroup(tctx->lp_ctx);
 	r.in.domain_name = &name;
 	r.out.info = &info;
 
 	connect_r.in.system_name = 0;
 	connect_r.in.access_mask = SEC_FLAG_MAXIMUM_ALLOWED;
 	connect_r.out.connect_handle = &handle;
-	
-	printf("Testing Connect and OpenDomain on BUILTIN\n");
 
-	status = dcerpc_samr_Connect(p, tctx, &connect_r);
-	if (!NT_STATUS_IS_OK(status)) {
-		if (NT_STATUS_EQUAL(status, NT_STATUS_ACCESS_DENIED)) {
-			printf("Connect failed (expected, schannel mapped to anonymous): %s\n",
-			       nt_errstr(status));
+	torture_comment(tctx, "Testing Connect and OpenDomain on BUILTIN\n");
+
+	torture_assert_ntstatus_ok(tctx, dcerpc_samr_Connect_r(b, tctx, &connect_r),
+		"Connect failed");
+	if (!NT_STATUS_IS_OK(connect_r.out.result)) {
+		if (NT_STATUS_EQUAL(connect_r.out.result, NT_STATUS_ACCESS_DENIED)) {
+			torture_comment(tctx, "Connect failed (expected, schannel mapped to anonymous): %s\n",
+			       nt_errstr(connect_r.out.result));
 		} else {
-			printf("Connect failed - %s\n", nt_errstr(status));
+			torture_comment(tctx, "Connect failed - %s\n", nt_errstr(connect_r.out.result));
 			return false;
 		}
 	} else {
@@ -160,22 +266,24 @@ static bool test_samr_ops(struct torture_context *tctx,
 		opendom.in.access_mask = SEC_FLAG_MAXIMUM_ALLOWED;
 		opendom.in.sid = dom_sid_parse_talloc(tctx, "S-1-5-32");
 		opendom.out.domain_handle = &domain_handle;
-		
-		status = dcerpc_samr_OpenDomain(p, tctx, &opendom);
-		if (!NT_STATUS_IS_OK(status)) {
-			printf("OpenDomain failed - %s\n", nt_errstr(status));
+
+		torture_assert_ntstatus_ok(tctx, dcerpc_samr_OpenDomain_r(b, tctx, &opendom),
+			"OpenDomain failed");
+		if (!NT_STATUS_IS_OK(opendom.out.result)) {
+			torture_comment(tctx, "OpenDomain failed - %s\n", nt_errstr(opendom.out.result));
 			return false;
 		}
 	}
 
-	printf("Testing GetDomPwInfo with name %s\n", r.in.domain_name->string);
-	
+	torture_comment(tctx, "Testing GetDomPwInfo with name %s\n", r.in.domain_name->string);
+
 	/* do several ops to test credential chaining */
 	for (i=0;i<5;i++) {
-		status = dcerpc_samr_GetDomPwInfo(p, tctx, &r);
-		if (!NT_STATUS_IS_OK(status)) {
-			if (!NT_STATUS_EQUAL(status, NT_STATUS_ACCESS_DENIED)) {
-				printf("GetDomPwInfo op %d failed - %s\n", i, nt_errstr(status));
+		torture_assert_ntstatus_ok(tctx, dcerpc_samr_GetDomPwInfo_r(b, tctx, &r),
+			"GetDomPwInfo failed");
+		if (!NT_STATUS_IS_OK(r.out.result)) {
+			if (!NT_STATUS_EQUAL(r.out.result, NT_STATUS_ACCESS_DENIED)) {
+				torture_comment(tctx, "GetDomPwInfo op %d failed - %s\n", i, nt_errstr(r.out.result));
 				return false;
 			}
 		}
@@ -191,35 +299,34 @@ static bool test_samr_ops(struct torture_context *tctx,
 static bool test_lsa_ops(struct torture_context *tctx, struct dcerpc_pipe *p)
 {
 	struct lsa_GetUserName r;
-	NTSTATUS status;
 	bool ret = true;
 	struct lsa_String *account_name_p = NULL;
 	struct lsa_String *authority_name_p = NULL;
+	struct dcerpc_binding_handle *b = p->binding_handle;
 
-	printf("\nTesting GetUserName\n");
+	torture_comment(tctx, "\nTesting GetUserName\n");
 
-	r.in.system_name = "\\";	
+	r.in.system_name = "\\";
 	r.in.account_name = &account_name_p;
 	r.in.authority_name = &authority_name_p;
 	r.out.account_name = &account_name_p;
 
 	/* do several ops to test credential chaining and various operations */
-	status = dcerpc_lsa_GetUserName(p, tctx, &r);
+	torture_assert_ntstatus_ok(tctx, dcerpc_lsa_GetUserName_r(b, tctx, &r),
+		"lsa_GetUserName failed");
 
 	authority_name_p = *r.out.authority_name;
 
-	if (NT_STATUS_EQUAL(status, NT_STATUS_RPC_PROTSEQ_NOT_SUPPORTED)) {
-		printf("not considering %s to be an error\n", nt_errstr(status));
-	} else if (!NT_STATUS_IS_OK(status)) {
-		printf("GetUserName failed - %s\n", nt_errstr(status));
+	if (!NT_STATUS_IS_OK(r.out.result)) {
+		torture_comment(tctx, "GetUserName failed - %s\n", nt_errstr(r.out.result));
 		return false;
 	} else {
 		if (!r.out.account_name) {
 			return false;
 		}
-		
+
 		if (strcmp(account_name_p->string, "ANONYMOUS LOGON") != 0) {
-			printf("GetUserName returned wrong user: %s, expected %s\n",
+			torture_comment(tctx, "GetUserName returned wrong user: %s, expected %s\n",
 			       account_name_p->string, "ANONYMOUS LOGON");
 			/* FIXME: gd */
 			if (!torture_setting_bool(tctx, "samba3", false)) {
@@ -229,9 +336,9 @@ static bool test_lsa_ops(struct torture_context *tctx, struct dcerpc_pipe *p)
 		if (!authority_name_p || !authority_name_p->string) {
 			return false;
 		}
-		
+
 		if (strcmp(authority_name_p->string, "NT AUTHORITY") != 0) {
-			printf("GetUserName returned wrong user: %s, expected %s\n",
+			torture_comment(tctx, "GetUserName returned wrong user: %s, expected %s\n",
 			       authority_name_p->string, "NT AUTHORITY");
 			/* FIXME: gd */
 			if (!torture_setting_bool(tctx, "samba3", false)) {
@@ -239,10 +346,6 @@ static bool test_lsa_ops(struct torture_context *tctx, struct dcerpc_pipe *p)
 			}
 		}
 	}
-	if (!test_many_LookupSids(p, tctx, NULL)) {
-		printf("LsaLookupSids3 failed!\n");
-		return false;
-	}
 
 	return ret;
 }
@@ -267,24 +370,25 @@ static bool test_schannel(struct torture_context *tctx,
 	struct dcerpc_pipe *p_lsa = NULL;
 	struct netlogon_creds_CredentialState *creds;
 	struct cli_credentials *credentials;
+	enum dcerpc_transport_t transport;
 
-	join_ctx = torture_join_domain(tctx, 
-				       talloc_asprintf(tctx, "%s%d", TEST_MACHINE_NAME, i), 
+	join_ctx = torture_join_domain(tctx,
+				       talloc_asprintf(tctx, "%s%d", TEST_MACHINE_NAME, i),
 				       acct_flags, &credentials);
 	torture_assert(tctx, join_ctx != NULL, "Failed to join domain");
 
 	status = dcerpc_parse_binding(tctx, binding, &b);
 	torture_assert_ntstatus_ok(tctx, status, "Bad binding string");
 
-	b->flags &= ~DCERPC_AUTH_OPTIONS;
-	b->flags |= dcerpc_flags;
+	status = dcerpc_binding_set_flags(b, dcerpc_flags, DCERPC_AUTH_OPTIONS);
+	torture_assert_ntstatus_ok(tctx, status, "set flags");
 
 	status = dcerpc_pipe_connect_b(tctx, &p, b, &ndr_table_samr,
 				       credentials, tctx->ev, tctx->lp_ctx);
-	torture_assert_ntstatus_ok(tctx, status, 
-		"Failed to connect with schannel");
+	torture_assert_ntstatus_ok(tctx, status,
+		"Failed to connect to samr with schannel");
 
-	torture_assert(tctx, test_samr_ops(tctx, p), 
+	torture_assert(tctx, test_samr_ops(tctx, p->binding_handle),
 		       "Failed to process schannel secured SAMR ops");
 
 	/* Also test that when we connect to the netlogon pipe, that
@@ -295,20 +399,24 @@ static bool test_schannel(struct torture_context *tctx,
 	status = dcerpc_epm_map_binding(tctx, b, &ndr_table_netlogon, tctx->ev, tctx->lp_ctx);
 	torture_assert_ntstatus_ok(tctx, status, "epm map");
 
-	status = dcerpc_secondary_connection(p, &p_netlogon, 
+	status = dcerpc_secondary_connection(p, &p_netlogon,
 					     b);
-	torture_assert_ntstatus_ok(tctx, status, "seconday connection");
+	torture_assert_ntstatus_ok(tctx, status, "secondary connection");
 
-	status = dcerpc_bind_auth(p_netlogon, &ndr_table_netlogon, 
-				  credentials, lp_gensec_settings(tctx, tctx->lp_ctx),
+	status = dcerpc_bind_auth(p_netlogon, &ndr_table_netlogon,
+				  credentials, lpcfg_gensec_settings(tctx, tctx->lp_ctx),
 				  DCERPC_AUTH_TYPE_SCHANNEL,
 				  dcerpc_auth_level(p->conn),
 				  NULL);
 
 	torture_assert_ntstatus_ok(tctx, status, "bind auth");
 
-	status = dcerpc_schannel_creds(p_netlogon->conn->security_state.generic_state, tctx, &creds);
-	torture_assert_ntstatus_ok(tctx, status, "schannel creds");
+	creds = cli_credentials_get_netlogon_creds(credentials);
+	torture_assert(tctx, (creds != NULL), "schannel creds");
+
+	/* checks the capabilities */
+	torture_assert(tctx, test_netlogon_capabilities(p_netlogon, tctx, credentials, creds),
+		       "Failed to process schannel secured capability ops (on fresh connection)");
 
 	/* do a couple of logins */
 	torture_assert(tctx, test_netlogon_ops(p_netlogon, tctx, credentials, creds),
@@ -317,68 +425,93 @@ static bool test_schannel(struct torture_context *tctx,
 	torture_assert(tctx, test_netlogon_ex_ops(p_netlogon, tctx, credentials, creds),
 		"Failed to process schannel secured NETLOGON EX ops");
 
+	/* we *MUST* use ncacn_np for openpolicy etc. */
+	transport = dcerpc_binding_get_transport(b);
+	status = dcerpc_binding_set_transport(b, NCACN_NP);
+	torture_assert_ntstatus_ok(tctx, status, "set transport");
+
 	/* Swap the binding details from SAMR to LSARPC */
 	status = dcerpc_epm_map_binding(tctx, b, &ndr_table_lsarpc, tctx->ev, tctx->lp_ctx);
 	torture_assert_ntstatus_ok(tctx, status, "epm map");
 
-	status = dcerpc_secondary_connection(p, &p_lsa, 
-					     b);
+	torture_assert_ntstatus_ok(tctx,
+		dcerpc_pipe_connect_b(tctx, &p_lsa, b, &ndr_table_lsarpc,
+				      credentials, tctx->ev, tctx->lp_ctx),
+		"failed to connect lsarpc with schannel");
 
-	torture_assert_ntstatus_ok(tctx, status, "seconday connection");
-
-	status = dcerpc_bind_auth(p_lsa, &ndr_table_lsarpc,
-				  credentials, lp_gensec_settings(tctx, tctx->lp_ctx),
-				  DCERPC_AUTH_TYPE_SCHANNEL,
-				  dcerpc_auth_level(p->conn),
-				  NULL);
-
-	torture_assert_ntstatus_ok(tctx, status, "bind auth");
-
-	torture_assert(tctx, test_lsa_ops(tctx, p_lsa), 
+	torture_assert(tctx, test_lsa_ops(tctx, p_lsa),
 		"Failed to process schannel secured LSA ops");
 
+	talloc_free(p_lsa);
+	p_lsa = NULL;
+
+	/* we *MUST* use ncacn_ip_tcp for lookupsids3/lookupnames4 */
+	status = dcerpc_binding_set_transport(b, NCACN_IP_TCP);
+	torture_assert_ntstatus_ok(tctx, status, "set transport");
+
+	torture_assert_ntstatus_ok(tctx,
+		dcerpc_epm_map_binding(tctx, b, &ndr_table_lsarpc, tctx->ev, tctx->lp_ctx),
+		"failed to call epm map");
+
+	torture_assert_ntstatus_ok(tctx,
+		dcerpc_pipe_connect_b(tctx, &p_lsa, b, &ndr_table_lsarpc,
+				      credentials, tctx->ev, tctx->lp_ctx),
+		"failed to connect lsarpc with schannel");
+
+	torture_assert(tctx,
+		test_many_LookupSids(p_lsa, tctx, NULL),
+		"LsaLookupSids3 failed!\n");
+
+	status = dcerpc_binding_set_transport(b, transport);
+	torture_assert_ntstatus_ok(tctx, status, "set transport");
+
+
 	/* Drop the socket, we want to start from scratch */
 	talloc_free(p);
 	p = NULL;
 
 	/* Now see what we are still allowed to do */
-	
+
 	status = dcerpc_parse_binding(tctx, binding, &b);
 	torture_assert_ntstatus_ok(tctx, status, "Bad binding string");
 
-	b->flags &= ~DCERPC_AUTH_OPTIONS;
-	b->flags |= dcerpc_flags;
+	status = dcerpc_binding_set_flags(b, dcerpc_flags, DCERPC_AUTH_OPTIONS);
+	torture_assert_ntstatus_ok(tctx, status, "set flags");
 
 	status = dcerpc_pipe_connect_b(tctx, &p_samr2, b, &ndr_table_samr,
 				       credentials, tctx->ev, tctx->lp_ctx);
-	torture_assert_ntstatus_ok(tctx, status, 
+	torture_assert_ntstatus_ok(tctx, status,
 		"Failed to connect with schannel");
 
 	/* do a some SAMR operations.  We have *not* done a new serverauthenticate */
-	torture_assert (tctx, test_samr_ops(tctx, p_samr2), 
+	torture_assert (tctx, test_samr_ops(tctx, p_samr2->binding_handle),
 			"Failed to process schannel secured SAMR ops (on fresh connection)");
 
 	/* Swap the binding details from SAMR to NETLOGON */
 	status = dcerpc_epm_map_binding(tctx, b, &ndr_table_netlogon, tctx->ev, tctx->lp_ctx);
 	torture_assert_ntstatus_ok(tctx, status, "epm");
 
-	status = dcerpc_secondary_connection(p_samr2, &p_netlogon2, 
+	status = dcerpc_secondary_connection(p_samr2, &p_netlogon2,
 					     b);
-	torture_assert_ntstatus_ok(tctx, status, "seconday connection");
+	torture_assert_ntstatus_ok(tctx, status, "secondary connection");
 
 	/* and now setup an SCHANNEL bind on netlogon */
 	status = dcerpc_bind_auth(p_netlogon2, &ndr_table_netlogon,
-				  credentials, lp_gensec_settings(tctx, tctx->lp_ctx),
+				  credentials, lpcfg_gensec_settings(tctx, tctx->lp_ctx),
 				  DCERPC_AUTH_TYPE_SCHANNEL,
 				  dcerpc_auth_level(p_samr2->conn),
 				  NULL);
 
 	torture_assert_ntstatus_ok(tctx, status, "auth failed");
-	
+
+	/* checks the capabilities */
+	torture_assert(tctx, test_netlogon_capabilities(p_netlogon2, tctx, credentials, creds),
+		       "Failed to process schannel secured capability ops (on fresh connection)");
+
 	/* Try the schannel-only SamLogonEx operation */
-	torture_assert(tctx, test_netlogon_ex_ops(p_netlogon2, tctx, credentials, creds), 
+	torture_assert(tctx, test_netlogon_ex_ops(p_netlogon2, tctx, credentials, creds),
 		       "Failed to process schannel secured NETLOGON EX ops (on fresh connection)");
-		
+
 
 	/* And the more traditional style, proving that the
 	 * credentials chaining state is fully present */
@@ -389,7 +522,8 @@ static bool test_schannel(struct torture_context *tctx,
 	talloc_free(p_samr2);
 
 	/* We don't want schannel for this test */
-	b->flags &= ~DCERPC_AUTH_OPTIONS;
+	status = dcerpc_binding_set_flags(b, 0, DCERPC_AUTH_OPTIONS);
+	torture_assert_ntstatus_ok(tctx, status, "set flags");
 
 	status = dcerpc_pipe_connect_b(tctx, &p_netlogon3, b, &ndr_table_netlogon,
 				       credentials, tctx->ev, tctx->lp_ctx);
@@ -421,19 +555,26 @@ bool torture_rpc_schannel(struct torture_context *torture)
 		uint16_t acct_flags;
 		uint32_t dcerpc_flags;
 	} tests[] = {
-		{ ACB_WSTRUST,   DCERPC_SCHANNEL | DCERPC_SIGN},
-		{ ACB_WSTRUST,   DCERPC_SCHANNEL | DCERPC_SEAL},
+		{ ACB_WSTRUST,   DCERPC_SCHANNEL | DCERPC_SIGN | DCERPC_SCHANNEL_AUTO},
+		{ ACB_WSTRUST,   DCERPC_SCHANNEL | DCERPC_SEAL | DCERPC_SCHANNEL_AUTO},
 		{ ACB_WSTRUST,   DCERPC_SCHANNEL | DCERPC_SIGN | DCERPC_SCHANNEL_128},
 		{ ACB_WSTRUST,   DCERPC_SCHANNEL | DCERPC_SEAL | DCERPC_SCHANNEL_128 },
-		{ ACB_SVRTRUST,  DCERPC_SCHANNEL | DCERPC_SIGN },
-		{ ACB_SVRTRUST,  DCERPC_SCHANNEL | DCERPC_SEAL },
+		{ ACB_WSTRUST,   DCERPC_SCHANNEL | DCERPC_SIGN | DCERPC_SCHANNEL_AES},
+		{ ACB_WSTRUST,   DCERPC_SCHANNEL | DCERPC_SEAL | DCERPC_SCHANNEL_AES },
+		{ ACB_SVRTRUST,  DCERPC_SCHANNEL | DCERPC_SIGN | DCERPC_SCHANNEL_AUTO},
+		{ ACB_SVRTRUST,  DCERPC_SCHANNEL | DCERPC_SEAL | DCERPC_SCHANNEL_AUTO},
 		{ ACB_SVRTRUST,  DCERPC_SCHANNEL | DCERPC_SIGN | DCERPC_SCHANNEL_128 },
-		{ ACB_SVRTRUST,  DCERPC_SCHANNEL | DCERPC_SEAL | DCERPC_SCHANNEL_128 }
+		{ ACB_SVRTRUST,  DCERPC_SCHANNEL | DCERPC_SEAL | DCERPC_SCHANNEL_128 },
+		{ ACB_SVRTRUST,  DCERPC_SCHANNEL | DCERPC_SIGN | DCERPC_SCHANNEL_AES },
+		{ ACB_SVRTRUST,  DCERPC_SCHANNEL | DCERPC_SEAL | DCERPC_SCHANNEL_AES }
 	};
 	int i;
 
 	for (i=0;i<ARRAY_SIZE(tests);i++) {
-		if (!test_schannel(torture, 
+		torture_comment(torture, "Testing with acct_flags=0x%x dcerpc_flags=0x%x \n",
+		       tests[i].acct_flags, tests[i].dcerpc_flags);
+
+		if (!test_schannel(torture,
 				   tests[i].acct_flags, tests[i].dcerpc_flags,
 				   i)) {
 			torture_comment(torture, "Failed with acct_flags=0x%x dcerpc_flags=0x%x \n",
@@ -458,22 +599,22 @@ bool torture_rpc_schannel2(struct torture_context *torture)
 	struct cli_credentials *credentials1, *credentials2;
 	uint32_t dcerpc_flags = DCERPC_SCHANNEL | DCERPC_SIGN;
 
-	join_ctx = torture_join_domain(torture, talloc_asprintf(torture, "%s2", TEST_MACHINE_NAME), 
+	join_ctx = torture_join_domain(torture, talloc_asprintf(torture, "%s2", TEST_MACHINE_NAME),
 				       ACB_WSTRUST, &credentials1);
-	torture_assert(torture, join_ctx != NULL, 
+	torture_assert(torture, join_ctx != NULL,
 		       "Failed to join domain with acct_flags=ACB_WSTRUST");
 
-	credentials2 = (struct cli_credentials *)talloc_memdup(torture, credentials1, sizeof(*credentials1));
-	credentials1->netlogon_creds = NULL;
-	credentials2->netlogon_creds = NULL;
+	credentials2 = cli_credentials_shallow_copy(torture, credentials1);
+	cli_credentials_set_netlogon_creds(credentials1, NULL);
+	cli_credentials_set_netlogon_creds(credentials2, NULL);
 
 	status = dcerpc_parse_binding(torture, binding, &b);
 	torture_assert_ntstatus_ok(torture, status, "Bad binding string");
 
-	b->flags &= ~DCERPC_AUTH_OPTIONS;
-	b->flags |= dcerpc_flags;
+	status = dcerpc_binding_set_flags(b, dcerpc_flags, DCERPC_AUTH_OPTIONS);
+	torture_assert_ntstatus_ok(torture, status, "set flags");
 
-	printf("Opening first connection\n");
+	torture_comment(torture, "Opening first connection\n");
 	status = dcerpc_pipe_connect_b(torture, &p1, b, &ndr_table_netlogon,
 				       credentials1, torture->ev, torture->lp_ctx);
 	torture_assert_ntstatus_ok(torture, status, "Failed to connect with schannel");
@@ -483,8 +624,8 @@ bool torture_rpc_schannel2(struct torture_context *torture)
 				       credentials2, torture->ev, torture->lp_ctx);
 	torture_assert_ntstatus_ok(torture, status, "Failed to connect with schannel");
 
-	credentials1->netlogon_creds = NULL;
-	credentials2->netlogon_creds = NULL;
+	cli_credentials_set_netlogon_creds(credentials1, NULL);
+	cli_credentials_set_netlogon_creds(credentials2, NULL);
 
 	torture_comment(torture, "Testing logon on pipe1\n");
 	if (!test_netlogon_ex_ops(p1, torture, credentials1, NULL))
@@ -554,7 +695,7 @@ static void torture_schannel_bench_connected(struct composite_context *c)
 	}
 }
 
-static void torture_schannel_bench_recv(struct rpc_request *req);
+static void torture_schannel_bench_recv(struct tevent_req *subreq);
 
 static bool torture_schannel_bench_start(struct torture_schannel_bench_conn *conn)
 {
@@ -562,7 +703,7 @@ static bool torture_schannel_bench_start(struct torture_schannel_bench_conn *con
 	NTSTATUS status;
 	DATA_BLOB names_blob, chal, lm_resp, nt_resp;
 	int flags = CLI_CRED_NTLM_AUTH;
-	struct rpc_request *req;
+	struct tevent_req *subreq;
 	struct cli_credentials *user_creds;
 
 	if (conn->total % 2) {
@@ -571,11 +712,11 @@ static bool torture_schannel_bench_start(struct torture_schannel_bench_conn *con
 		user_creds = s->user2_creds;
 	}
 
-	if (lp_client_lanman_auth(s->tctx->lp_ctx)) {
+	if (lpcfg_client_lanman_auth(s->tctx->lp_ctx)) {
 		flags |= CLI_CRED_LANMAN_AUTH;
 	}
 
-	if (lp_client_ntlmv2_auth(s->tctx->lp_ctx)) {
+	if (lpcfg_client_ntlmv2_auth(s->tctx->lp_ctx)) {
 		flags |= CLI_CRED_NTLMv2_AUTH;
 	}
 
@@ -593,7 +734,7 @@ static bool torture_schannel_bench_start(struct torture_schannel_bench_conn *con
 	chal = data_blob_const(conn->ninfo.challenge,
 			       sizeof(conn->ninfo.challenge));
 
-	names_blob = NTLMv2_generate_names_blob(conn->tmp, 
+	names_blob = NTLMv2_generate_names_blob(conn->tmp,
 						cli_credentials_get_workstation(conn->wks_creds),
 						cli_credentials_get_domain(conn->wks_creds));
 
@@ -619,7 +760,7 @@ static bool torture_schannel_bench_start(struct torture_schannel_bench_conn *con
 
 	conn->r.in.server_name = talloc_asprintf(conn->tmp, "\\\\%s", dcerpc_server_name(conn->pipe));
 	conn->r.in.computer_name = cli_credentials_get_workstation(conn->wks_creds);
-	conn->r.in.logon_level = 2;
+	conn->r.in.logon_level = NetlogonNetworkInformation;
 	conn->r.in.logon = talloc(conn->tmp, union netr_LogonLevel);
 	conn->r.in.logon->network = &conn->ninfo;
 	conn->r.in.flags = talloc(conn->tmp, uint32_t);
@@ -628,24 +769,26 @@ static bool torture_schannel_bench_start(struct torture_schannel_bench_conn *con
 	conn->r.out.authoritative = talloc(conn->tmp, uint8_t);
 	conn->r.out.flags = conn->r.in.flags;
 
-	req = dcerpc_netr_LogonSamLogonEx_send(conn->pipe, conn->tmp, &conn->r);
-	torture_assert(s->tctx, req, "Failed to setup LogonSamLogonEx request");
+	subreq = dcerpc_netr_LogonSamLogonEx_r_send(s, s->tctx->ev,
+						    conn->pipe->binding_handle,
+						    &conn->r);
+	torture_assert(s->tctx, subreq, "Failed to setup LogonSamLogonEx request");
 
-	req->async.callback = torture_schannel_bench_recv;
-	req->async.private_data = conn;
+	tevent_req_set_callback(subreq, torture_schannel_bench_recv, conn);
 
 	return true;
 }
 
-static void torture_schannel_bench_recv(struct rpc_request *req)
+static void torture_schannel_bench_recv(struct tevent_req *subreq)
 {
 	bool ret;
 	struct torture_schannel_bench_conn *conn =
-		(struct torture_schannel_bench_conn *)req->async.private_data;
+		(struct torture_schannel_bench_conn *)tevent_req_callback_data_void(subreq);
 	struct torture_schannel_bench *s = talloc_get_type(conn->s,
 					   struct torture_schannel_bench);
 
-	s->error = dcerpc_ndr_request_recv(req);
+	s->error = dcerpc_netr_LogonSamLogonEx_r_recv(subreq, subreq);
+	TALLOC_FREE(subreq);
 	if (!NT_STATUS_IS_OK(s->error)) {
 		return;
 	}
@@ -684,16 +827,12 @@ bool torture_rpc_schannel_bench1(struct torture_context *torture)
 	s->nprocs = torture_setting_int(torture, "nprocs", 4);
 	s->conns = talloc_zero_array(s, struct torture_schannel_bench_conn, s->nprocs);
 
-	s->user1_creds = (struct cli_credentials *)talloc_memdup(s,
-								 cmdline_credentials,
-								 sizeof(*s->user1_creds));
+	s->user1_creds = cli_credentials_shallow_copy(s, cmdline_credentials);
 	tmp = torture_setting_string(s->tctx, "extra_user1", NULL);
 	if (tmp) {
 		cli_credentials_parse_string(s->user1_creds, tmp, CRED_SPECIFIED);
 	}
-	s->user2_creds = (struct cli_credentials *)talloc_memdup(s,
-								 cmdline_credentials,
-								 sizeof(*s->user1_creds));
+	s->user2_creds = cli_credentials_shallow_copy(s, cmdline_credentials);
 	tmp = torture_setting_string(s->tctx, "extra_user2", NULL);
 	if (tmp) {
 		cli_credentials_parse_string(s->user1_creds, tmp, CRED_SPECIFIED);
@@ -712,21 +851,24 @@ bool torture_rpc_schannel_bench1(struct torture_context *torture)
 	cli_credentials_set_kerberos_state(s->wks_creds2, CRED_DONT_USE_KERBEROS);
 
 	for (i=0; i < s->nprocs; i++) {
+		struct cli_credentials *wks = s->wks_creds1;
+
+		if ((i % 2) && (torture_setting_bool(torture, "multijoin", false))) {
+			wks = s->wks_creds2;
+		}
+
 		s->conns[i].s = s;
 		s->conns[i].index = i;
-		s->conns[i].wks_creds = (struct cli_credentials *)talloc_memdup(
-			s->conns, s->wks_creds1,sizeof(*s->wks_creds1));
-		if ((i % 2) && (torture_setting_bool(torture, "multijoin", false))) {
-			memcpy(s->conns[i].wks_creds, s->wks_creds2,
-			       talloc_get_size(s->conns[i].wks_creds));
-		}
-		s->conns[i].wks_creds->netlogon_creds = NULL;
+		s->conns[i].wks_creds = cli_credentials_shallow_copy(s->conns, wks);
+		cli_credentials_set_netlogon_creds(s->conns[i].wks_creds, NULL);
 	}
 
 	status = dcerpc_parse_binding(s, binding, &s->b);
 	torture_assert_ntstatus_ok(torture, status, "Bad binding string");
-	s->b->flags &= ~DCERPC_AUTH_OPTIONS;
-	s->b->flags |= DCERPC_SCHANNEL | DCERPC_SIGN;
+
+	status = dcerpc_binding_set_flags(s->b, DCERPC_SCHANNEL | DCERPC_SIGN,
+					  DCERPC_AUTH_OPTIONS);
+	torture_assert_ntstatus_ok(torture, status, "set flags");
 
 	torture_comment(torture, "Opening %d connections in parallel\n", s->nprocs);
 	for (i=0; i < s->nprocs; i++) {
@@ -756,8 +898,8 @@ bool torture_rpc_schannel_bench1(struct torture_context *torture)
 	}
 
 	while (NT_STATUS_IS_OK(s->error) && s->nprocs != s->nconns) {
-		int ev_ret = event_loop_once(torture->ev);
-		torture_assert(torture, ev_ret == 0, "event_loop_once failed");
+		int ev_ret = tevent_loop_once(torture->ev);
+		torture_assert(torture, ev_ret == 0, "tevent_loop_once failed");
 #endif
 	}
 	torture_assert_ntstatus_ok(torture, s->error, "Failed establish a connect");
@@ -770,7 +912,7 @@ bool torture_rpc_schannel_bench1(struct torture_context *torture)
 
 	{
 		struct netr_ServerPasswordSet pwset;
-		char *password = generate_random_str(s->join_ctx1, 8);
+		char *password = generate_random_password(s->join_ctx1, 8, 255);
 		struct netlogon_creds_CredentialState *creds_state;
 		struct dcerpc_pipe *net_pipe;
 		struct netr_Authenticator credential, return_authenticator;
@@ -802,13 +944,14 @@ bool torture_rpc_schannel_bench1(struct torture_context *torture)
 		netlogon_creds_des_encrypt(creds_state, &new_password);
 		netlogon_creds_client_authenticator(creds_state, &credential);
 
-		status = dcerpc_netr_ServerPasswordSet(net_pipe, torture, &pwset);
-		torture_assert_ntstatus_ok(torture, status,
+		torture_assert_ntstatus_ok(torture, dcerpc_netr_ServerPasswordSet_r(net_pipe->binding_handle, torture, &pwset),
+			"ServerPasswordSet failed");
+		torture_assert_ntstatus_ok(torture, pwset.out.result,
 					   "ServerPasswordSet failed");
 
 		if (!netlogon_creds_client_check(creds_state,
 					&pwset.out.return_authenticator->cred)) {
-			printf("Credential chaining failed\n");
+			torture_comment(torture, "Credential chaining failed\n");
 		}
 
 		cli_credentials_set_password(s->wks_creds1, password,
@@ -818,8 +961,7 @@ bool torture_rpc_schannel_bench1(struct torture_context *torture)
 
 		/* Just as a test, connect with the new creds */
 
-		talloc_free(s->wks_creds1->netlogon_creds);
-		s->wks_creds1->netlogon_creds = NULL;
+		cli_credentials_set_netlogon_creds(s->wks_creds1, NULL);
 
 		status = dcerpc_pipe_connect_b(s, &net_pipe, s->b,
 					       &ndr_table_netlogon,
@@ -843,8 +985,8 @@ bool torture_rpc_schannel_bench1(struct torture_context *torture)
 	end = timeval_add(&start, s->timelimit, 0);
 
 	while (NT_STATUS_IS_OK(s->error) && !timeval_expired(&end)) {
-		int ev_ret = event_loop_once(torture->ev);
-		torture_assert(torture, ev_ret == 0, "event_loop_once failed");
+		int ev_ret = tevent_loop_once(torture->ev);
+		torture_assert(torture, ev_ret == 0, "tevent_loop_once failed");
 	}
 	torture_assert_ntstatus_ok(torture, s->error, "Failed some request");
 	s->stopped = true;
